
#ifndef __ALGORITHMAN__REGULAREXPRESSION__REGULAREXPRESSION_H
#define __ALGORITHMAN__REGULAREXPRESSION__REGULAREXPRESSION_H

#include <string>
#include <iostream>
#include "automaton.h"

namespace AlgorithMan
{
    namespace RegularExpressions
    {

        class RegularExpression
        {
            public:
                RegularExpression();
                virtual ~RegularExpression();
                virtual std::string UnixSyntax() = 0;
                virtual AlgorithMan::Automaton* ThompsonConstruction() = 0;
                static RegularExpression* Parse(std::istream& is);
        };


        class Sum : public RegularExpression
        {
            private:
                RegularExpression *A;
                RegularExpression *B;

            public:
                Sum(RegularExpression *A, RegularExpression *B);
                virtual ~Sum();
                virtual std::string UnixSyntax();
                virtual AlgorithMan::Automaton* ThompsonConstruction();
        };


        class Product : public RegularExpression
        {
            private:
                RegularExpression *A;
                RegularExpression *B;
            public:
                Product(RegularExpression *A, RegularExpression *B);
                virtual ~Product();
                virtual std::string UnixSyntax();
                virtual AlgorithMan::Automaton* ThompsonConstruction();
        };


        class KleeneStar : public RegularExpression
        {
            private:
                RegularExpression *A;
            public:
                KleeneStar(RegularExpression *A);
                virtual ~KleeneStar();
                virtual std::string UnixSyntax();
                virtual AlgorithMan::Automaton* ThompsonConstruction();
        };


        class FixedText : public RegularExpression
        {
            private:
                std::string *str;
            public:
                FixedText(std::string *str);
                virtual ~FixedText();
                virtual std::string UnixSyntax();
                virtual AlgorithMan::Automaton* ThompsonConstruction();
        };
    }
}

#endif
